mata

/**************************************************************************/
/*                                                                        */
/*                                                                        */
/*   bvopmvop(...): mata function to estimate a multivariate ordered      */
/*        probit model via oprobit, biprobit, and suest.                  */
/*                                                                        */
/*                                                                        */
/*   bvopmvop(ynames,xnames,ifc,quivar,rnd,printres) takes six arguments: */
/*                                                                        */
/*     1. ynames: a 1x1 string containing the names of the M>=2           */
/*          ordered outcomes, e.g. yn ="y1 y2 y3 y4"                      */
/*                                                                        */
/*     2. xnames: a 1x1 string containing the names of the (k-1)          */
/*          covariates (a constant is added automatically),               */
/*          e.g. xn="age educ sex region price"                           */
/*                                                                        */
/*     3. ifc: a 1x1 string containing any "if" conditions for subset     */
/*          estimation, e.g. ic="if female==1 & income>10000"             */
/*          Set equal to "" for no subsetting.                            */
/*                                                                        */
/*     4. quivar: Use quivar=1 if display of the results of the bivariate */
/*          probit and suest estimates is to be suppressed, =0 else       */
/*                                                                        */
/*     5. rnd: Specify rounding of displayed results,                     */
/*          e.g. rndvar=.0001                                             */
/*                                                                        */
/*     6. printres: Specify whether (=1) or not (=0) the estimates of the */
/*          averaged beta-hats and correlation matrix are displayed.      */
/*          If printres=1, the output will also display "cut & paste"     */
/*          versions of the estimated beta, cutpoint, and correlation     */
/*          matrixes.  These may be useful for input into subsequent      */
/*          Mata computations.                                            */
/*                                                                        */
/*                                                                        */
/*     Examples:   bv1=bvopmvop(yn,xn,ic,0,.0001,1)                       */
/*                 bv2=bvopmvop("y1 y2 y3","x1 x2 x3","",1,.001,0)        */
/*                                                                        */
/*                                                                        */
/*  bvopmvop(...) returns the px(p+1) matrix, whose first column contains */
/*   the stacked vector containing the beta-hats, the cutpoints, and the  */
/*   atanh of the estimated correlation matrix (excluding the diagonal    */
/*   elements), i.e.:                                                     */
/*                                                                        */
/*         t=[b1',...,bM',c1',...,cM',athrho(1,2),...,athrho((M-1),M)]'   */
/*                                                                        */
/*   and whose remaining pxp matrix is the estimated vcov(t).             */
/*                                                                        */
/*   Note: The program accommodates outcomes y(j) having different        */
/*         numbers of categories G(j) across j=1,...,M, including some    */
/*         of the y(j) being binary.                                      */
/*                                                                        */
/*   Note: This function uses Stata's suest to combine estimates.         */
/*         Since suest can accommodate combinations of estimates that are */
/*         based on different estimation samples, missingness of          */
/*         data on outcomes (and covariates) across observations is       */
/*         permissible.                                                   */
/*                                                                        */
/*   Note: Executing this mata function clears any stored estimates.      */
/*                                                                        */
/*   Note: For large problems it may be necessary to adjust Stata's       */
/*         matsize using "set matsize..."  This is so suest does not      */
/*         encounter size limitations when computing the combined         */
/*         parameter vector and its vcov matrix.  It is possible that     */
/*         some very large problems cannot be handled due to Stata's      */
/*         internal constraint of matsize<=11,000.                        */
/*                                                                        */
/**************************************************************************/


function bvopmvop(ynames,xnames,ifc,quivar,rndvar,printres) {

/****** Declarations and Initializations ***/

real r,m,k,rnd,sub,suv,sus,rind,rsel,tall,vall,rtmp,sn,ocmat,ocsmat,rselb,rselc,rsela,ctemp,ns1a,ytmp,xtmp,cuts,cc1,cc2,stmp

string xn,xn1,yn,xt,yt,sp,eno,enb,qv,rmat,rp,lp,sun,ns1,ns2,blmat,icv,obmat,obsmat,ocp,ocsp,rs,rdis,nlcs,cm,cpmb,cpmr,bamat


xn1=xnames
sp=" "
lp="("
rp=")"
cm=","
xn=xnames
yn=ynames
eno="eo"
enb="eb"
if (quivar==0) qv=""
else qv="qui "
xt=tokens(xn)
yt=tokens(yn)
m=cols(yt)
k=cols(xt)
r=I(m)
rs=J(m,m," ")
rnd=rndvar
icv=ifc
nlcs="exp(2*_b[athrho:_cons])"


/****** Estimate Univariate Ordered Probits; Store Results ***/

stata("estimates clear")
cuts=J(1,m,0) 
for (i=1; i<=m; i++) {
    stata(qv+"oprobit "+yt[1,i]+sp+xn1+sp+icv)
    stata(qv+"estimates store "+eno+strofreal(i))
    cuts[i]=st_numscalar("e(k_cat)")-1
} 


/****** Estimate Bivariate Probits; Store Results; Compute R Matrix ***/
  
for (i=1; i<=(m-1); i++) {
  for (j=(i+1); j<=m; j++)   {
    stata(qv+"biprobit "+yt[1,i]+sp+yt[1,j]+sp+xn1+sp+icv)
    stata(qv+"estimates store "+enb+strofreal(i)+strofreal(j))
    stata("qui nlcom ("+nlcs+"-1)/("+nlcs+"+1), post")
    rtmp=st_matrix("e(b)")
    stmp=sqrt(st_matrix("e(V)"))
    r[i,j]=rtmp
    r[j,i]=r[i,j]
    rs[i,j]=lp+strofreal(round(stmp,rnd))+rp
    rs[j,i]=rs[i,j]
  }
} 


/****** Combine Estimates Using suest; Retrieve Combined Estimates ***/

stata(qv+"suest "+eno+"*"+sp+enb+"*")
sun=st_matrixcolstripe("e(b)")
sub=st_matrix("e(b)")
suv=st_matrix("e(V)")
sus=sqrt(diagonal(suv))
sn=st_numscalar("e(N)")
ns1=sun[.,1]
ns2=sun[.,2]


/****** Construct Selection Matrix for B Estimates ***/ 

rselb=J(0,rows(sun),.)
ns1a=strpos(ns1,eno)
for (i=1; i<=m; i++) {
  ytmp=strpos(ns1,yt[i])
  for (j=1; j<=k; j++) {
    xtmp=strpos(ns2,xt[j])
    rselb=rselb\((ns1a:*ytmp:*xtmp):>0)'
  }
}


/****** Construct Selection Matrix for C Estimates ***/ 

ctemp=selectindex(strpos(ns1,"cut"):>0)
rselc=J(rows(ctemp),rows(sun),0)
for (i=1; i<=rows(ctemp); i++) {
  rselc[i,ctemp[i,.]]=1
}


/****** Construct Selection Matrix for ATHRHO Estimates ***/ 

rsela=J((.5*m*(m-1)),rows(sun),0)
rind=selectindex(strpos(ns1,"athrho"):>0)
for (i=1; i<=(.5*m*(m-1)); i++) {
  rsela[i,rind[i,.]]=1
}


/****** Select B and C Elements and Their Standard Errors for Display ***/

if (printres==1) {
  obmat=strofreal(round(colshape(rselb*sub',k)',rnd))
  obsmat=(lp:+strofreal(round(colshape(rselb*sus,k)',rnd)):+rp)    
  ocmat=rselc*sub'
  ocsmat=rselc*sus
  ocp=J(max(cuts),m,"--")
  ocsp=J(max(cuts),m," ")
  cc2=cuts
  for (i=2;i<=cols(cuts);i++) {
    cc2[i]=cc2[i]+cc2[i-1]
  }
  cc1=(1,(cc2:+1))[1..cols(cc2)]
  for (i=1;i<=m;i++) {
    ocp[1..cuts[i],i]=strofreal(round(ocmat[cc1[i]..cc2[i]],rnd))
    ocsp[1..cuts[i],i]=lp:+strofreal(round(ocsmat[cc1[i]..cc2[i]],rnd)):+rp
  }
}


/****** Compute Full Parameter Estimate Vector & Associated vcov Matrix ***/

rsel=rselb\rselc\rsela
tall=rsel*sub'
vall=rsel*suv*rsel'


/****** Display Results ***/

if (printres==1) {
  blmat=J(1,(m+1)," ")
  rmat=(" ",yt)\blmat
  for (i=1; i<=k; i++) {
    rmat=rmat\(xt[.,i],obmat[i,.])
    rmat=rmat\(sp,obsmat[i,.])
    rmat=rmat\blmat
  }

  for (i=1; i<=rows(ocp); i++) {
    rmat=rmat\(("cut"+strofreal(i)),ocp[i,.])
    rmat=rmat\(sp,ocsp[i,.])
    rmat=rmat\blmat
  }

  " "
  "******************************************************"
  "*                                                    *"
  "*       Multivariate Ordered Probit: Results         *"
  "*                                                    *"
  "******************************************************"
  " "
  "N. of Observations (from suest): "+strofreal(sn)
  " "
  if (icv != "") {
    "Estimation Sample: "+icv
  }
  " "
  " "
  "Beta-Hat and Cutpoint Point Estimates and Estimated Standard Errors"
  "   (Note: SEs are from suest ests.)"
  " "
  rmat
  " "
  " "
  "Estimated Correlation (Rho) Matrix and Estimated Standard Errors"
  " "
  rdis=(" ",yt)\J(1,(m+1)," ")
  for (i=1; i<=m; i++) {
    rdis=rdis\(yt[.,i],strofreal(round(r[i,.],rnd)))
    rdis=rdis\(" ",rs[i,.])
    rdis=rdis\J(1,(m+1)," ")
  }
  rdis

  bamat=obmat\ocp
  bamat=(bamat:!="--"):*bamat+(bamat:=="--"):*"."
  cpmb=sp:+sp:+lp:+bamat[.,1]
  for (i=2;i<=cols(bamat);i++) {
    cpmb=cpmb:+sp:+cm:+sp:+bamat[.,i]
  }
  cpmb=cpmb:+rp:+sp:+(J(rows(bamat)-1,1,"\")\sp)

  " "
  " "
  "Cut & Paste Matrix, Beta-Hat and Cutpoint Point Estimates"
  " "
  display(cpmb)
  " "
  " "
  cpmr=sp:+sp:+lp:+strofreal(round(r[.,1],rnd))
  for (i=2;i<=cols(r);i++) {
    cpmr=cpmr:+sp:+cm:+sp:+strofreal(round(r[.,i],rnd))
  }
  cpmr=cpmr:+rp:+sp:+(J(rows(r)-1,1,"\")\sp)

  "Cut & Paste Matrix, Estimated Correlation Matrix"
  " "
  display(cpmr)
  " "
  " "
}

stata("estimates clear")

return((tall,vall))
}


end



